@@ -208,7 +208,7 @@ function s = sym(x, varargin)
     return
 
   elseif (isinteger(x)  &&  nargin==1)
-    s = sym(num2str(x));
+    s = sym(num2str(x, '%ld'));
     return
 
   elseif (isa (x, 'double')  &&  nargin==1)
@@ -269,9 +269,6 @@ function s = sym(x, varargin)
 
 
   elseif (isa (x, 'char'))
-    useSymbolNotS = false;
-    cmd = [];
-
     asm = [];
     if (nargin == 2 && isequal(size(varargin{1}), [1 2]))
       s = make_sym_matrix(x, varargin{1});
@@ -279,53 +276,58 @@ function s = sym(x, varargin)
     elseif (nargin >= 2)
       % assume the remaining inputs are assumptions
       asm = varargin;
-      useSymbolNotS = true;
     end
 
     doDecimalCheck = true;
 
-    % various special cases for x
-    if (strcmp(x, 'pi'))
-      cmd = 'z = sp.pi';
-    elseif (strcmpi(x, 'inf')) || (strcmpi(x, '+inf'))
-      cmd = 'z = sp.oo';
+    % preprocess
+    if (strcmpi(x, 'inf')) || (strcmpi(x, '+inf'))
+      x = 'oo';
     elseif (strcmpi(x, '-inf'))
-      cmd = 'z = -sp.oo';
-    elseif (strcmpi(x, 'nan'))
-      cmd = 'z = sp.nan';
+      x = '-oo';
     elseif (strcmpi(x, 'i'))
-      cmd = 'z = sp.I';
-    %% Symbols with special meanings in SymPy: Issue #23
-    elseif (strcmp(x, 'beta') || strcmp(x, 'gamma') || ...
-            strcmp(x, 'zeta') || strcmp(x, 'Chi') || ...
-            strcmp(x, 'S') || strcmp(x, 'N') || strcmp(x, 'Q'))
-      useSymbolNotS = true;
+      x = 'I';
+    elseif (strcmpi(x, '-i'))
+      x = '-I';
+    elseif (strcmpi(x, 'nan'))
+      x = 'nan';
     elseif (strcmp(x, 'lambda'))
       x = 'lamda';
-      useSymbolNotS = true;
     elseif (strcmp(x, 'Lambda'))
       x = 'Lamda';
+    end
+
+    % Decide whether to pass to S() or Symbol()
+    if (any(strcmp(x, {'pi', 'I', 'oo', 'zoo', 'nan'})))
+      useSymbolNotS = false;
+    elseif (regexp(x, '^-?\d*\.?\d*(e-?\d+)?$'))
+      % Numbers: integers and floats
+      useSymbolNotS = false;
+    elseif (regexp(x, '^\w+$'))
+      % Words.  Note must follow numbers case.
+      % Use Symbol instead of S, e.g., for Issue #23:
+      % strcmp(x, {'beta' 'gamma' 'zeta' 'Chi' 'E' 'E1' 'Ei' 'S' 'N' 'Q'})
+      % But we also expect sym('Eq') to work, so match all single words
       useSymbolNotS = true;
     elseif (~isempty (strfind (x, '(') ))
-      %disp('debug: has a "(", not a symfun, assuming srepr!')
+      % SymPy "srepr" or other raw python code
       useSymbolNotS = false;
       doDecimalCheck = false;
     else
-      %disp(['debug: just a regular symbol: ' x])
+      % Other non-symbols such as sym('1/3')
+      useSymbolNotS = false;
     end
 
     if (~useSymbolNotS)
-      % if we're not forcing Symbol() then we use S(), unless
-      % cmd already set.
-      if (isempty(cmd))
-        if (doDecimalCheck && ~isempty(strfind(x, '.')))
-          warning('possibly unintended decimal point in constructor string');
-        end
-        % x is raw sympy, could have various quotes in it
-        cmd = sprintf('z = sympy.S("%s")', strrep(x, '"', '\"'));
+      % Use S(), as we're not forcing Symbol()
+      assert (isempty (asm))   % sym('pi', 'integer')
+      if (doDecimalCheck && ~isempty(strfind(x, '.')))
+        warning('possibly unintended decimal point in constructor string');
       end
+      % x is raw sympy, could have various quotes in it
+      cmd = sprintf('z = sympy.S("%s")', strrep(x, '"', '\"'));
+
     else % useSymbolNotS
-      assert(isempty(cmd), 'inconsistent input')
       if (isempty(asm))
         cmd = sprintf('z = sympy.Symbol("%s")', x);
 
@@ -615,6 +617,14 @@ end
 %! s = sym(a);
 %! assert (isequal (double(a), [1 2 100]))
 
+%!xtest
+%! % bigger int64 integer types: TODO: passes on Octave 4?
+%! q = int64(123456789012345);
+%! w = 10000*q + 123;
+%! a = sym('1234567890123450123');
+%! b = sym(w);
+%! assert (isequal (a, b))
+
 %!test
 %! % sym(double) heuristic
 %! s = warning ('off', 'OctSymPy:sym:rationalapprox');
@@ -626,6 +636,17 @@ end
 %! assert (isequal (x/sym(pi), sym(1)/123))
 %! warning (s)
 
+%!test
+%! % symbols with special sympy names
+%! syms Ei Eq
+%! assert (regexp(char(Eq), '^Symbol'))
+%! assert (regexp(char(Ei), '^Symbol'))
+
+%!test
+%! % E can be a sym not just exp(sym(1))
+%! syms E
+%! assert (~logical (E == exp(sym(1))))
+
 %!error <assumption is not supported>
 %! x = sym('x', 'positive2');
 
@@ -635,6 +656,12 @@ end
 %!error <assumption is not supported>
 %! x = sym('x', 'integer2', 'positive');
 
+%!error <failed>
+%! x = sym('-pi', 'positive')
+
+%!error <failed>
+%! x = sym('pi', 'integer')
+
 %!xtest
 %! % multiple assumptions
 %! % FIXME: xtest for sympy <= 0.7.6 where a is the full dict
